/*
 *  zmalloc.c
 *		
 *
 *
 *  Copyright (C) 1990	Lysator Computer Club,
 *			Linkoping University,  Sweden
 *
 *  Everyone is granted permission to copy, modify and redistribute
 *  this code, provided the people they give it to can.
 *
 *
 *  Author:	Thomas Bellman
 *		Lysator Computer Club
 *		Linkoping University
 *		Sweden
 *
 *  email:	Bellman@Lysator.LiU.SE
 *
 *
 *  Any opinions expressed in this code are the author's PERSONAL opinions,
 *  and does NOT, repeat NOT, represent any official standpoint of Lysator,
 *  even if so stated.
 */



#include <stddef.h>
#include <malloc.h>
#include <string.h>

#include <compiler.h>

#include "zmalloc.h"


#define EXPORT


typedef	struct	{
	int		refcount;
	size_t		size;
}  z_info;



#define	REFCOUNT(pointer)	\
	(((z_info *) ((char *) pointer - sizeof (z_info)))->refcount)

#define	AREA_SIZE(pointer)	\
	(((z_info *) ((char *) pointer - sizeof (z_info)))->size)


#define MIN(n1, n2)		(((n1) < (n2)) ? (n1) : (n2))



EXPORT  void *
zmalloc (size_t	   size)
{
    void		* memory;

    memory = malloc (size + sizeof (z_info));
    if (memory == NULL)
	return  NULL;
    ((z_info *) memory)->size = size;
    ((z_info *) memory)->refcount = 1;
    return  (char *) memory + sizeof (z_info);
}



EXPORT  void *
zcalloc (size_t	   n,
	 size_t	   size)

{
    void		* memory;

    memory = malloc (n * size + sizeof (z_info));
    if (memory == NULL)
	return  NULL;
    ((z_info *) memory)->size = n * size;
    ((z_info *) memory)->refcount = 1;
    memset ((char *) memory + sizeof (z_info),
	    0, n * size);

    return  (char *) memory + sizeof (z_info);
}



EXPORT  void *
zrealloc (void	  * ptr,
	  size_t    size)

{
    void		* memory;

    if (ptr == NULL)
	return  zmalloc (size);
    if (REFCOUNT (ptr) > 1)
    {
	memory = zmalloc (size);
	if (memory == NULL)
	    return  NULL;
	memcpy (memory, ptr, MIN (size, AREA_SIZE (ptr)));
	--REFCOUNT (ptr);
	return  memory;
    }
    else
    {
	memory = realloc ((char *) ptr - sizeof (z_info),
			  size + sizeof (z_info));
	return  (char *) memory + sizeof (z_info);
    }
}



EXPORT  void
zfree (void	* ptr)
{
    if (ptr != NULL && --REFCOUNT (ptr) <= 0)
    {
	free ((char *) ptr - sizeof (z_info));
    }
}



EXPORT  void
zdestruct (void	  * ptr)
{
    if (ptr != NULL)
	free ((char *) ptr - sizeof (z_info));
}



EXPORT  void *
zuse (void	* ptr)
{
    if (ptr != NULL)
    {
	++REFCOUNT (ptr);
    }
    return  ptr;
}



EXPORT  void *
zown (void	* ptr)
{
    void	* memory;

    if (ptr == NULL)
	return  NULL;
    if (REFCOUNT (ptr) == 1)
	return  ptr;
    else
    {
	if ((memory = malloc (AREA_SIZE (ptr))) == NULL)
	    return  NULL;
	memcpy (memory, ptr, AREA_SIZE (ptr));
	zfree (ptr);

	return  memory;
    }
}
