/*
** mux.h
*/

#ifndef __LYSKOM__MUX_H__
#define __LYSKOM__MUX_H__

#define MUX_OUTMSGSIZE	   1024

#define MUX_MAGIC_ALLOC    0x56AE17DB
#define MUX_MAGIC_FREE     0xF24A6BE7

typedef enum
{
  MUX_TYPE_CLIENT,
  MUX_TYPE_MUX
} Mux_type;



typedef struct
{
  int             id;     /* MUX client id */
  Connection    * conn;   /* LysKOM virtual Connection */

  struct isc_msg * outmsg;
} Mux_client;

typedef struct
{
    int			parse_pos;
    int			parse_pos_2;
    int			string_parse_pos;
    int			function; /* Function to call. */

    /* Gather data in these variables. */
    
    long		num;
    String		string;
    String              unparsed;
    String_size		first_to_parse; /* Index into unparsed. */
    Bool		more_to_parse;	/* Any chance that there is anything
					   left in unparsed? */
} Mux_parse;

typedef struct mux
{
  unsigned long     magic;    /* Magic number :-) */
  Mux_type          type;
  struct isc_scb  * scb;

  Mux_parse         parse;

  int               client_c;
  Mux_client      * client_v;
} Mux;


/*
** MUX support functions
*/

extern Mux *
mux_create(Mux_type type, IscSession *scb);
  
extern void
mux_destruct(Mux *mp);

extern int
mux_clients(Mux *mp);

extern void
mux_addclient(Mux *mp, int id, Connection *cp);

extern void
mux_delclient(Mux *mp, int id);

extern Mux_client *
mux_getclientbyid(Mux *mp, int id);

extern Mux_client *
mux_getclientbyconn(Mux *mp, Connection *cp);



/*
** MUX output functions
*/

extern int
mux_write(Connection  * cp,
	  const char  * buf,
	  size_t        size);

extern int
mux_printf(Connection  * cp,
	   const char  * format,
	   ...);

extern int
mux_putc(int           chr,
	 Connection  * cp);

extern void
mux_flush(Connection  * cp);

extern void
mux_close(Connection  * cp);

extern void
mux_free_parsed(Mux *mp);

#endif
